@@ -1,6 +1,6 @@
 /* 
    sitecopy WebDAV protocol driver module
-   Copyright (C) 2000, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 2000-2005, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -18,7 +18,7 @@
 
 */
 
-#include <config.h>
+#include "config.h"
 
 #include <sys/types.h>
 
@@ -49,10 +49,12 @@
 #include <ne_uri.h>
 #include <ne_auth.h>
 #include <ne_dates.h>
+#include <ne_socket.h>
 
 #include "protocol.h"
 #include "frontend.h"
 #include "i18n.h"
+#include "common.h"
 
 struct fetch_context {
     struct proto_file **files;
@@ -73,14 +75,13 @@ struct private {
     int iscollection;
 };
 
-#define ELM_resourcetype (NE_ELM_PROPS_UNUSED)
-#define ELM_collection (NE_ELM_PROPS_UNUSED + 1)
+#define ELM_resourcetype (NE_PROPS_STATE_TOP + 1)
+#define ELM_collection (NE_PROPS_STATE_TOP + 2)
 
 /* The element definitinos for the complex prop handler. */
-static const struct ne_xml_elm fetch_elms[] = {
-    { "DAV:", "resourcetype", ELM_resourcetype, 0 },
-    { "DAV:", "collection", ELM_collection, 0 },
-    { NULL }
+static const struct ne_xml_idmap fetch_elms[] = {
+    { "DAV:", "resourcetype", ELM_resourcetype },
+    { "DAV:", "collection", ELM_collection },
 };
 
 static const ne_propname props[] = {
@@ -99,32 +100,16 @@ static inline int get_depth(const char *href) {
     return count;
 }
 
-/* TODO: get rid of this? */
-static void set_err(ne_session *sess, const char *msg)
+/* Set session error string to 'msg: strerror(errnum)'. */
+static void syserr(ne_session *sess, const char *msg, int errnum)
 {
-    char *err;
-    CONCAT2(err, msg, strerror(errno));
-    ne_set_error(sess, err);
-    free(err);
+    char err[256];
+    ne_set_error(sess, "%s: %s", msg, ne_strerror(errnum, err, sizeof err));
 }
 
 static int get_server_port(struct site *site)
 {
-    const char *svc;
-    int port, defport;
-    if (site->http_secure) {
-	svc = "https";
-        defport = 443;
-    } else {
-	svc = "http";
-	defport = 80;
-    }
-    port = sock_service_lookup(svc);
-    if (port == 0) {
-	port = defport;
-	NE_DEBUG(NE_DBG_HTTP, "Using default port for %s: %d\n", svc, port);
-    }
-    return port;
+    return site->http_secure ? 443 : 80;
 }
 
 static int get_proxy_port(struct site *site)
@@ -185,7 +170,7 @@ static int h2s(ne_session *sess, int errcode)
 	return SITE_OK;
     case NE_AUTH:
 	return SITE_AUTH;
-    case NE_AUTHPROXY:
+    case NE_PROXYAUTH:
 	return SITE_PROXYAUTH;
     case NE_FAILED:
 	return SITE_FAILED;
@@ -196,83 +181,91 @@ static int h2s(ne_session *sess, int errcode)
     case NE_TIMEOUT:
 	ne_set_error(sess, _("The connection timed out."));
 	return SITE_ERRORS;
-    case NE_SERVERAUTH:
-	ne_set_error(sess, 
-		       _("The server did not authenticate itself correctly.\n"
-			 "Report this error to your server administrator."));
-	return SITE_ERRORS;
-    case NE_PROXYAUTH:
-	ne_set_error(sess, 
-		 _("The proxy server did not authenticate itself correctly.\n"
-		   "Report this error to your proxy server administrator."));
-	return SITE_ERRORS;
     case NE_ERROR:
     default:
 	return SITE_ERRORS;
     }
 }
 
-/* Special h2s() wrapper for init() functions. */
-static int h2s_init(ne_session *sess, int errcode)
+/* Verify callback for when a trusted cert is not known. */
+static int verify_untrusted(void *userdata, int failures,
+                            const ne_ssl_certificate *cert)
 {
-    int ret = h2s(sess, errcode);
-    if (ret == SITE_ERRORS)
-	ret = SITE_FAILED;
-    return ret;
+    struct site *site = userdata;
+
+    if (fe_accept_cert(cert, failures))
+        return -1;
+    
+    /* TODO: how to handle a write error here? */
+    ne_ssl_cert_write(cert, site->certfile);
+
+    return 0;
 }
 
-static int 
-init(void **session, struct site *site)
+/* Verify callback for when a trusted cert is known. */
+static int verify_trusted(void *userdata, int failures,
+                          const ne_ssl_certificate *cert)
 {
-    ne_session *sess = ne_session_create();
+    ne_ssl_certificate *expected = userdata;
+
+    return failures != NE_SSL_UNTRUSTED || ne_ssl_cert_cmp(expected, cert);
+}
+
+static int init(void **session, struct site *site)
+{
+    ne_session *sess;
     ne_server_capabilities caps = {0};
     int ret;
+    char *root;
+
+    sess = ne_session_create(site->http_secure?"https":"http",
+			     site->server.hostname, site->server.port);
 
     *session = sess;
 
-    ne_set_status(sess, notify_cb, NULL);
+    if (site->http_secure && !ne_has_support(NE_FEATURE_SSL)) {
+	ne_set_error(sess, _("SSL support has not be compiled in."));
+	return SITE_FAILED;
+    }
 
-    if (site->http_use_expect)
- 	ne_set_expect100(sess, 1);
-    else
-	ne_set_expect100(sess, 0);
+    if (site->http_secure) {
+        if (access(site->certfile, R_OK) == 0) {
+            ne_ssl_certificate *cert = ne_ssl_cert_read(site->certfile);
+            if (cert == NULL) {
+                ne_set_error(sess, _("Could not load certificate `%s'."),
+                             site->certfile);
+                return SITE_FAILED;
+            }
+            ne_ssl_set_verify(sess, verify_trusted, cert);
+        } else {
+            ne_ssl_set_verify(sess, verify_untrusted, site);
+        }
+    }
+
+    ne_set_status(sess, notify_cb, NULL);
 
     if (site->http_limit)
 	ne_set_persist(sess, 0);
 
-    if (site->http_secure) {
-	if (ne_set_secure(sess, 1)) {
-	    ne_set_error(sess, _("SSL support has not be compiled in."));
-	    return SITE_FAILED;
-	}
-    }
-
     /* Note, this won't differentiate between xsitecopy and
      * sitecopy... maybe we should put a comment in as well. */
-    ne_set_useragent(sess, PACKAGE "/" VERSION);
+    ne_set_useragent(sess, PACKAGE_NAME "/" PACKAGE_VERSION);
 
     if (site->proxy.hostname) {
 	ne_set_proxy_auth(sess, proxy_auth_cb, &site->proxy);
-	ret = ne_session_proxy(sess, site->proxy.hostname, site->proxy.port);
-	if (ret == NE_LOOKUP)
-	    return SITE_PROXYLOOKUP;
-	else if (ret != NE_OK) {
-	    return h2s_init(sess, ret);
-	}
+	ne_session_proxy(sess, site->proxy.hostname, site->proxy.port);
     }
 
     ne_set_server_auth(sess, server_auth_cb, &site->server);
-    ret = ne_session_server(sess, site->server.hostname, site->server.port);
-    if (ret != NE_OK) {
-	return h2s_init(sess, ret);
-    }
     
     if (site->http_tolerant) {
 	/* Skip the OPTIONS, since we ignore failure anyway. */
 	return SITE_OK;
     }
 
-    ret = ne_options(sess, site->remote_root, &caps);
+    root = ne_path_escape(site->remote_root);
+    ret = ne_options(sess, root, &caps);
+    ne_free(root);
     if (ret == NE_OK) {
 	if (!caps.dav_class1) {
 	    ne_set_error(sess, 
@@ -285,7 +278,10 @@ init(void **session, struct site *site)
 	    return SITE_FAILED;
 	}
     } else {
-	return h2s_init(sess, ret);
+        ret = h2s(sess, ret);
+        if (ret == SITE_ERRORS)
+            ret = SITE_FAILED;
+        return ret;
     }
 
 
@@ -304,8 +300,8 @@ static int file_move(void *session, const char *from, const char *to)
     char *efrom, *eto;
     int ret;
 
-    efrom = uri_abspath_escape(from);
-    eto = uri_abspath_escape(to);
+    efrom = ne_path_escape(from);
+    eto = ne_path_escape(to);
 
     /* Always overwrite destination. */
     ret = ne_move(sess, 1, efrom, eto);
@@ -324,11 +320,11 @@ static int file_upload(void *session, const char *local, const char *remote,
     char *eremote;
 
     if (fd < 0) {
-	set_err(sess, _("Could not open file: "));
+	syserr(sess, _("Could not open file"), errno);
 	return SITE_ERRORS;
     }
     
-    eremote = uri_abspath_escape(remote);
+    eremote = ne_path_escape(remote);
     ENABLE_PROGRESS;
     ret = ne_put(sess, eremote, fd);
     DISABLE_PROGRESS;
@@ -339,6 +335,49 @@ static int file_upload(void *session, const char *local, const char *remote,
     return h2s(sess, ret);
 }
 
+/* conditional PUT using If-Unmodified-Since. */
+static int put_if_unmodified(ne_session *sess, const char *uri, int fd,
+                             time_t since)
+{
+    ne_request *req = ne_request_create(sess, "PUT", uri);
+    char *date = ne_rfc1123_date(since);
+    int ret;
+
+    /* Add in the conditional header */
+    ne_add_request_header(req, "If-Unmodified-Since", date);
+    ne_free(date);
+    
+#if NE_VERSION_MINOR == 24
+    ne_set_request_body_fd(req, fd);
+#else
+    {
+        struct stat st;
+
+        if (fstat(fd, &st) < 0) {
+            int errnum = errno;
+            ne_set_error(sess, _("Could not stat file: %s"), strerror(errnum));
+            return NE_ERROR;
+        }
+        
+        ne_set_request_body_fd(req, fd, 0, st.st_size);
+    }
+#endif
+
+    ret = ne_request_dispatch(req);
+    
+    if (ret == NE_OK) {
+	if (ne_get_status(req)->code == 412) {
+	    ret = NE_FAILED;
+	} else if (ne_get_status(req)->klass != 2) {
+	    ret = NE_ERROR;
+	}
+    }
+
+    ne_request_destroy(req);
+
+    return ret;
+}
+
 static int 
 file_upload_cond(void *session, const char *local, const char *remote,
 		 int ascii, time_t t)
@@ -348,13 +387,13 @@ file_upload_cond(void *session, const char *local, const char *remote,
     char *eremote;
 
     if (fd < 0) {
-	set_err(sess, _("Could not open file: "));
+	syserr(sess, _("Could not open file"), errno);
 	return SITE_ERRORS;
     }
     
-    eremote = uri_abspath_escape(remote);
+    eremote = ne_path_escape(remote);
     ENABLE_PROGRESS;
-    ret = h2s(sess, ne_put_if_unmodified(sess, eremote, fd, t));
+    ret = h2s(sess, put_if_unmodified(sess, eremote, fd, t));
     DISABLE_PROGRESS;
     free(eremote);
     
@@ -369,7 +408,7 @@ static int file_get_modtime(void *session, const char *remote, time_t *modtime)
     int ret;
     char *eremote;
  
-    eremote = uri_abspath_escape(remote);
+    eremote = ne_path_escape(remote);
     ret = ne_getmodtime(sess,eremote,modtime);
     free(eremote);
 
@@ -380,15 +419,16 @@ static int file_download(void *session, const char *local, const char *remote,
 			 int ascii) 
 {
     ne_session *sess = session;
-    int ret, fd = open(local, O_CREAT | O_WRONLY | OPEN_BINARY_FLAGS, 0644);
+    int ret, fd = open(local,
+		       O_TRUNC | O_CREAT | O_WRONLY | OPEN_BINARY_FLAGS, 0644);
     char *eremote;
 
     if (fd < 0) {
-	set_err(sess, _("Could not open file: "));
+	syserr(sess, _("Could not open file"), errno);
 	return SITE_ERRORS;
     }
 
-    eremote = uri_abspath_escape(remote);
+    eremote = ne_path_escape(remote);
     ENABLE_PROGRESS;
     ret = h2s(sess, ne_get(sess, eremote, fd));
     DISABLE_PROGRESS;
@@ -402,15 +442,28 @@ static int file_download(void *session, const char *local, const char *remote,
 }
 
 static int file_read(void *session, const char *remote, 
-		     sock_block_reader reader, void *userdata) 
+		     ne_block_reader reader, void *userdata) 
 {
     ne_session *sess = session;
     int ret;
     char *eremote;
+    ssize_t bytes;
+    ne_request *req;
 
-    eremote = uri_abspath_escape(remote);    
+    eremote = ne_path_escape(remote);    
+    req = ne_request_create(sess, "GET", eremote);
     ENABLE_PROGRESS;
-    ret = ne_read_file(sess, eremote, reader, userdata);
+    do {
+	char buf[BUFSIZ];
+	ret = ne_begin_request(req);
+	if (ret != NE_OK) break;
+	while ((bytes = ne_read_response_block(req, buf, sizeof buf)) > 0)
+	    reader(userdata, buf, bytes);
+	if (bytes < 0)
+	    ret = NE_ERROR;
+	else
+	    ret = ne_end_request(req);
+    } while (ret == NE_RETRY);
     DISABLE_PROGRESS;
     free(eremote);
 
@@ -420,7 +473,7 @@ static int file_read(void *session, const char *remote,
 static int file_delete(void *session, const char *remote) 
 {
     ne_session *sess = session;
-    char *eremote = uri_abspath_escape(remote);
+    char *eremote = ne_path_escape(remote);
     int ret = ne_delete(sess, eremote);
 
     free(eremote);
@@ -436,7 +489,7 @@ static int file_chmod(void *session, const char *remote, mode_t mode)
     ne_proppatch_operation ops[] = { 
 	{ &execprop, ne_propset, NULL }, { NULL } 
     };
-    char *eremote = uri_abspath_escape(remote);
+    char *eremote = ne_path_escape(remote);
     int ret;
     
     /* True or false, depending... */
@@ -452,10 +505,21 @@ static int file_chmod(void *session, const char *remote, mode_t mode)
     return h2s(sess, ret);
 }
 
+/* Returns escaped path string for a directory. */
+static char *coll_escape(const char *dirname)
+{
+    char *ret = ne_path_escape(dirname);
+    if (!ne_path_has_trailing_slash(ret)) {
+	ret = ne_realloc(ret, strlen(ret) + 2);
+	strcat(ret, "/");
+    }
+    return ret;
+}
+
 static int dir_create(void *session, const char *dirname)
 {
     ne_session *sess = session;
-    char *edirname = uri_abspath_escape(dirname);
+    char *edirname = coll_escape(dirname);
     int ret = ne_mkcol(sess, edirname);
     free(edirname);
     return h2s(sess, ret);
@@ -465,7 +529,7 @@ static int dir_create(void *session, const char *dirname)
 static int dir_remove(void *session, const char *dirname)
 {
     ne_session *sess = session;
-    char *edirname = uri_abspath_escape(dirname);
+    char *edirname = coll_escape(dirname);
     int ret = ne_delete(sess, edirname);
     free(edirname);
     return h2s(sess, ret);
@@ -520,9 +584,9 @@ static void pfind_results(void *userdata, const char *href,
 	return;
     }
 
-    uhref = uri_unescape(href);
+    uhref = ne_path_unescape(href);
 
-    if (!uri_childof(ctx->root, uhref)) {
+    if (!ne_path_childof(ctx->root, uhref)) {
 	/* URI not a child of the root collection...  ignore this
 	 * resource */
 	NE_DEBUG(NE_DBG_HTTP, "outside root collection!\n");
@@ -559,7 +623,7 @@ static void pfind_results(void *userdata, const char *href,
 	file->type = proto_dir;
 
 	/* Strip the trailing slash if it has one. */
-	if (uri_has_trailing_slash(file->filename)) {
+	if (ne_path_has_trailing_slash(file->filename)) {
 	    file->filename[strlen(file->filename) - 1] = '\0';
 	}
 
@@ -579,34 +643,29 @@ static void pfind_results(void *userdata, const char *href,
 
 }
 
-static int check_context(void *ud, ne_xml_elmid parent, ne_xml_elmid child)
-{
-    if (parent == NE_ELM_prop && child == ELM_resourcetype)
-	return NE_XML_VALID;
-
-    if (parent == ELM_resourcetype && child == ELM_collection)
-	return NE_XML_VALID;
-
-    return NE_XML_DECLINE;
-}
-
-static int end_element(void *userdata, 
-		       const struct ne_xml_elm *elm, const char *cdata)
+static int start_element(void *userdata, int parent,
+                         const char *nspace, const char *name,
+                         const char **atts)
 {
     ne_propfind_handler *handler = userdata;
-    struct private *private = ne_propfind_current_private(handler);
+    struct private *priv = ne_propfind_current_private(handler);
+    int state;
 
-    if (elm->id == ELM_collection)
-	private->iscollection = 1;
+    state = ne_xml_mapid(fetch_elms, NE_XML_MAPLEN(fetch_elms), nspace, name);
+
+    if (parent == NE_207_STATE_PROP && state == ELM_resourcetype)
+        return ELM_resourcetype;
     
-    return 0;
+    if (parent == ELM_resourcetype && state == ELM_collection)
+        priv->iscollection = 1;
+
+    return NE_XML_DECLINE;
 }
 
 /* Creates the private structure. */
 static void *create_private(void *userdata, const char *uri)
 {
-    struct private *private = ne_calloc(sizeof *private);
-    return private;
+    return ne_calloc(sizeof(struct private));
 }
 
 /* TODO: optimize: only ask for lastmod + executable when we really
@@ -620,7 +679,7 @@ static int fetch_list(void *session, const char *dirname, int need_modtimes,
     int ret;
     struct fetch_context ctx;
     ne_propfind_handler *ph;
-    char *edirname = uri_abspath_escape(dirname);
+    char *edirname = ne_path_escape(dirname);
 
     ctx.root = dirname;
     ctx.files = files;
@@ -630,8 +689,7 @@ static int fetch_list(void *session, const char *dirname, int need_modtimes,
     ne_propfind_set_private(ph, create_private, NULL);
 
     /* Register the handler for the complex props. */
-    ne_xml_push_handler(ne_propfind_get_parser(ph), fetch_elms,
-			 check_context, NULL, end_element, ph);
+    ne_xml_push_handler(ne_propfind_get_parser(ph), start_element, NULL, NULL, ph);
 
     ret = ne_propfind_named(ph, props, pfind_results, &ctx);
 
