@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1998-2002, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1998-2005, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -312,26 +312,51 @@ file_retrieve_server(struct site_file *file, struct site *site, void *session)
     free(full_remote);
 }
 
+/* Create new directories and change permissions on existing directories. */
 static int update_create_directories(struct site *site, void *session)
 {
     struct site_file *current;
     int ret = 0;
 
     for_each_file(current, site) {
-	if ((current->type == file_dir) && (current->diff == file_new) && 
-	    (!site->only_marked || !current->marked)) {
-	    /* New dir! */
+	if ((current->type == file_dir) 
+            && (current->diff == file_new || current->diff == file_changed)) {
+	    /* New or changed directory! */
 	    char *full_remote;
+            int oret;
+
 	    if (!fe_can_update(current)) continue;
+
 	    full_remote = file_full_remote(&current->local, site);
-	    fe_updating(current);
-	    if (CALL(dir_create)(session, full_remote) != SITE_OK) {
-		fe_updated(current, false, DRIVER_ERR);
-		ret = 1;
-	    } else {
-		fe_updated(current, true, NULL);
-		file_uploaded(current, site);
-	    }
+
+            if (current->diff == file_new) {
+                fe_updating(current);
+                oret = CALL(dir_create)(session, full_remote);
+                if (oret != SITE_OK) {
+                    fe_updated(current, false, DRIVER_ERR);
+                } else {
+                    fe_updated(current, true, NULL);
+                }
+            } else {
+                oret = SITE_OK;
+            }
+
+            if (site->dirperms && oret == SITE_OK) {
+                fe_setting_perms(current);
+                oret = CALL(file_chmod)(session, full_remote,
+                                        current->local.mode);
+                if (oret == SITE_OK) {
+                    fe_set_perms(current, true, NULL);
+                } else {
+                    fe_set_perms(current, false, DRIVER_ERR);
+                }
+            }
+
+            if (oret != SITE_OK) {
+                ret = 1;
+            } else {
+                file_uploaded(current, site);
+            }
 	    free(full_remote);
 	}
     }
@@ -372,9 +397,8 @@ static int update_delete_files(struct site *site, void *session)
 	next = current->next;
 	/* Skip directories and links, and only do deleted files on
 	 * this pass */
-	if ((current->diff == file_deleted) &&
-	    (current->type == file_file) && 
-	    (!site->only_marked || current->marked)) {
+	if (current->diff == file_deleted &&
+	    current->type == file_file) {
 	    char *full_remote;
 	    if (!fe_can_update(current)) continue;
 	    full_remote = file_full_remote(&current->stored, site);
@@ -400,7 +424,7 @@ static int update_move_files(struct site *site, void *session)
     struct site_file *current;
     char *old_full_remote, *full_remote;
     for_each_file(current, site) {
-	if (current->diff != file_moved || (site->only_marked && !current->marked)) 
+	if (current->diff != file_moved) 
 	    continue;
 	full_remote = file_full_remote(&current->local, site);
 	/* The file has been moved */
@@ -435,11 +459,10 @@ static int update_files(struct site *site, void *session)
 	/* This loop only handles changed and new files, so
 	 * skip everything else. */
 
-	if ((current->type != file_file) || 
-	    (current->diff == file_deleted) ||
-	    (current->diff == file_moved) ||
-	    (current->diff == file_unchanged) || 
-	    (site->only_marked && !current->marked)) continue;
+	if (current->type != file_file
+	    || current->diff == file_deleted
+	    || current->diff == file_moved
+	    || current->diff == file_unchanged) continue;
 
 	full_local = file_full_local(&current->local, site);
 	full_remote = file_full_remote(&current->local, site);
@@ -500,7 +523,7 @@ static int update_files(struct site *site, void *session)
 		    file_uploaded(current, site);
 		    break;
 		}
-	    } else if (site->tempupload && (current->diff == file_changed)) {
+	    } else if (site->tempupload) {
 		/* Do temp file upload followed by a move */
 		char *temp_remote = temp_upload_filename(full_remote, site);
 		if (CALL(file_upload)(session, full_local, temp_remote,
@@ -566,7 +589,6 @@ static int update_delete_directories(struct site *site, void *session)
      * directories are deleted bottom up */
     for (current=site->files_tail; current!=NULL; current=prev) {
 	prev = current->prev;
-	if (site->only_marked && !current->marked) continue;
 	if ((current->type==file_dir) && (current->diff == file_deleted)) {
 	    char *full_remote;
 	    if (!fe_can_update(current)) continue;
@@ -594,9 +616,8 @@ static int update_links(struct site *site, void *session)
     for (current=site->files; current!=NULL; current=next) {
 	char *full_remote;
 	next = current->next;
-	if ((current->type != file_link) || 
-	    (site->only_marked && !current->marked))
-	    continue;
+	if (current->type != file_link) continue;
+
 	full_remote = file_full_remote(&current->local, site);
 	switch (current->diff) {
 	case file_new:
@@ -651,11 +672,10 @@ static void proto_seterror(struct site *site, void *session)
 
 const char *site_get_protoname(struct site *site) 
 {
-    if (site->protocol == siteproto_unknown) {
-	return "Unknown";
-    } else {
+    if (site->driver)
 	return site->driver->protocol_name;
-    }
+    else
+	return site->proto_string;
 }
 
 static int proto_init(struct site *site, void **session)
@@ -796,7 +816,7 @@ void site_read_local_state(struct site *site)
 	    
 	    if (full != NULL) free(full);
 
-	    CONCAT2(full, this, ent->d_name);
+	    full = ne_concat(this, ent->d_name, NULL);
 
 #ifdef __EMX__
 /* There are no symlinks under OS/2, use stat() instead */
@@ -878,7 +898,7 @@ void site_read_local_state(struct site *site)
 		    dirstack = realloc(dirstack, sizeof(char *) * dirmax);
 		}
 		/* Add it to the search stack */
-		CONCAT2(dirstack[dirtop], full, "/");
+		dirstack[dirtop] = ne_concat(full, "/", NULL);
 		dirtop++;
 	    } else {
 		NE_DEBUG(DEBUG_FILES, "something else.\n");
@@ -1035,11 +1055,19 @@ site_fetch_walk(struct site *site, struct proto_file *files)
 
 }
 
-static void
+static
+#if NE_VERSION_MINOR == 24
+void
+#else
+int
+#endif
 site_fetch_csum_read(void *userdata, const char *s, size_t len)
 {
     struct ne_md5_ctx *md5 = userdata;
     ne_md5_process_bytes(s, len, md5);
+#if NE_VERSION_MINOR != 24
+    return 0;
+#endif
 }
 
 /* Retrieve the remote checksum for all files */
@@ -1054,7 +1082,7 @@ site_fetch_checksum(struct proto_file *files, struct site *site, void *session)
 	if (file->type == proto_file && 
 	    !file_isexcluded(file->filename, site)) {
 	    char *full_remote;
-	    CONCAT2(full_remote, site->remote_root, file->filename);
+	    full_remote = ne_concat(site->remote_root, file->filename, NULL);
 	    ne_md5_init_ctx(&md5);
 	    fe_checksumming(file->filename);
 	    if (CALL(file_read)(session, full_remote, 
@@ -1254,7 +1282,8 @@ void site_destroy(struct site *site)
  * with the given diff type in the given site, using the given section
  * name. */
 static void site_flatlist_items(FILE *f, struct site *site,
-				 enum file_diff diff, const char *name) {
+                                enum file_diff diff, const char *name)
+{
     struct site_file *current;
     fprintf(f, "sectstart|%s", name);
     putc('\n', f);
@@ -1263,10 +1292,11 @@ static void site_flatlist_items(FILE *f, struct site *site,
 	    fprintf(f, "item|%s%s", file_name(current),
 		    (current->type==file_dir)?"/":"");
 	    if (current->diff == file_moved) {
-		fprintf(f, "|%s\n", current->stored.filename);
-	    } else {
-		putc('\n', f);
-	    }	    
+		fprintf(f, "|%s", current->stored.filename);
+	    }
+            if (current->ignore)
+                fputs("|ignored", f);
+            putc('\n', f);
 	}
     }
     fprintf(f, "sectend|%s\n", name);
@@ -1296,5 +1326,5 @@ void site_sock_progress_cb(void *userdata, off_t progress, off_t total)
 
 void fe_initialize(void)
 {
-    sock_init();
+    ne_sock_init();
 }
