@@ -1,6 +1,6 @@
 /* 
    sitecopy, for managing remote web sites.
-   Copyright (C) 1999-2000, Joe Orton <joe@manyfish.co.uk>
+   Copyright (C) 1999-2004, Joe Orton <joe@manyfish.co.uk>
                                                                      
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -253,7 +253,6 @@ struct site_file {
 
     enum file_type type:2;
     
-    unsigned int marked:1; /* whether the file is marked to be updated or not */
     unsigned int ignore:1; /* whether to ignore any changes to this file */
 
     /* Probably want to make the states into an array, so they can be
@@ -292,6 +291,7 @@ enum site_protocol_modes {
     siteproto_ftp,
     siteproto_dav,
     siteproto_rsh,
+    siteproto_sftp,
     siteproto_unknown
 };
 
@@ -348,10 +348,7 @@ struct site {
     struct site_host proxy;
 
     enum site_protocol_modes protocol;
-    char *proto_string; /* If protocol == siteproto_unknown,
-			 * proto_string is set to the string used
-			 * in the rcfile to identify the protocol.
-			 */
+    char *proto_string; /* protocol name used in rcfile. */
     const struct proto_driver *driver; /* the protocol driver routines */
 
     char *remote_root; /* root directory of site on server */
@@ -362,16 +359,19 @@ struct site {
     unsigned int local_isrel:1; /* is the local root directory relative to home dir */
 
     char *infofile;  /* local storage file in ~/.sitecopy/  */
+    char *certfile;  /* file in which cached SSL certificate is stored. */
     FILE *storage_file;  /* The file opened for the storage file */
 
     /* Options for the site */
     enum site_perm_modes perms; /* permissions maintenance mode */
+    int dirperms; /* directory permissions maintenance mode */
     enum site_symlink_modes symlinks; /* symlink handline mode */
 
     /* Protocol-driver specific options here */
     unsigned int ftp_pasv_mode:1;
     unsigned int ftp_echo_quit:1;
     unsigned int ftp_forcecd:1;
+    unsigned int ftp_use_cwd:1;
     unsigned int http_use_expect:1;
     unsigned int http_limit:1;
     unsigned int http_secure:1;
@@ -389,7 +389,6 @@ struct site {
 
     /* These are parameters to site_update really. */
     unsigned int keep_going:1; /* if true, keep going past errors in updates */
-    unsigned int only_marked:1; /* if true, only act on marked files in updates */
 
     unsigned int use_this:1; /* whether the site is being operated on - handy
 			      * for the console FE */
@@ -496,10 +495,6 @@ int site_verify(struct site *site, int *numremoved);
  * called during the update. fe_can_update may be called during the
  * update if site->prompting is set.
  *
- * Only files with marked==true will be updated if site->only_marked == true.
- * WARNING: DO NOT USE only_marked==true LIGHTLY. You might mess up
- * the remote site. 
- *
  * Returns:
  *   SITE_ERRORS if an error occurred which was reported using
  *     the fe_update_* functions. site->last_error is undefined.
